<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2019 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.1/1829459
 */

namespace Application\Helper;

/**
 * Class ArrayHelper, helps with some common array functions.
 * @package Application\Helper
 */
class ArrayHelper
{
    /**
     * This is primarily to help with arrays when we have integers values (or values
     * that look like integers) as key values pointing to some other structure. PHPs
     * array_merge does not cope with these very well.
     * Looks through the target value to find the source key and if found replaces that
     * key with the provided source value. If the source key is not found it will be
     * added to the target.
     * @param array $target         the target array
     * @param mixed $sourceKey      the source key
     * @param mixed $sourceValue    the source value
     * @return array the merged array.
     */
    public static function mergeSingle(array $target, $sourceKey, $sourceValue)
    {
        if ($sourceKey) {
            $stringKey = (string)$sourceKey;
            array_map(
                function ($targetKey, $value) use ($stringKey, $sourceValue, &$target) {
                    if ($targetKey === $stringKey) {
                        $target[$targetKey] = $sourceValue;
                    }
                },
                array_map('strval', array_keys($target)),
                $target
            );
            if (!isset($target[$stringKey])) {
                $target = $target + array($sourceKey => $sourceValue);
            }
        }
        return $target;
    }

    /**
     * @see ArrayHelper::mergeSingle()
     * Iterates every key in the source to replace/add that key into the target.
     * @param array $target the target array
     * @param array $source the target array
     * @return array the merged array.
     */
    public static function merge(array $target, array $source)
    {
        array_map(
            function ($sourceKey, $sourceValue) use (&$target) {
                $target = ArrayHelper::mergeSingle($target, $sourceKey, $sourceValue);
            },
            array_map('strval', array_keys($source)),
            $source
        );
        return $target;
    }
}
